@@ -90,9 +90,9 @@ test_hdrl_persistence_mkhilist(
     return list;
 }
 
-static char *
+static const char *
 test_hdrl_persistence_make_trapMEF(
-        char * fn,
+        const char * fn,
         int maxx,
         int maxy,
         cpl_type maxt,
@@ -103,9 +103,9 @@ test_hdrl_persistence_make_trapMEF(
         int fullwelly,
         cpl_type fullwellt,
         int tabrows,
-        char * tabc1n,
+        const char * tabc1n,
         cpl_type tabc1t,
-        char * tabc2n,
+        const char * tabc2n,
         cpl_type tabc2t)
 {
     cpl_image * max = cpl_image_new(maxx, maxy, maxt);
@@ -267,7 +267,7 @@ test_hdrl_persistence_verify_inputs(void)
     hdrl_imagelist * bad5_illums = test_hdrl_persistence_mkhilist(1, 4, 4, 0);
 
     /* traps input variations */
-    char * good_trap = test_hdrl_persistence_make_trapMEF("good_vi_trap.fits",
+    const char * good_trap = test_hdrl_persistence_make_trapMEF("good_vi_trap.fits",
             3, 3, CPL_TYPE_DOUBLE, 3, 3, CPL_TYPE_DOUBLE, 3, 3, CPL_TYPE_DOUBLE, 6,
             "TAU", CPL_TYPE_FLOAT, "NU", CPL_TYPE_FLOAT);
 /*
