@@ -545,9 +545,6 @@ const cpl_frame *qmost_dfs_setup_product_default (
  *
  * @par CPL error codes raised
  *   - CPL_ERROR_NULL_INPUT if one of the required inputs is NULL.
- *   - CPL_ERROR_TYPE_MISMATCH if qclist contains a property
- *     overriding one already populated from the input file, but the
- *     data type doesn't match.
  *
  * @par Output FITS Headers:
  *   - <b>EXTNAME</b>
@@ -567,6 +564,10 @@ cpl_propertylist *qmost_dfs_setup_product_extension_header (
     cpl_propertylist *qclist)
 {
     cpl_propertylist *outhdr = NULL;
+    int i, n, copyme;
+    const cpl_property *prop;
+    cpl_type type;
+    const char *name;
 
 #undef TIDY
 #define TIDY                                    \
@@ -607,16 +608,66 @@ cpl_propertylist *qmost_dfs_setup_product_extension_header (
     cpl_propertylist_set_comment(outhdr, "ESO DRS DUMMY",
                                  "True if extension is a dummy product");
 
-    /* Copy in QC */
+    /* Copy in QC, safely */
     if(qclist != NULL) {
-        if(cpl_propertylist_copy_property_regexp(outhdr,
-                                                 qclist,
-                                                 ".*",
-                                                 0) != CPL_ERROR_NONE) {
-            TIDY;
-            cpl_error_set_message(cpl_func, cpl_error_get_code(),
-                                  "could not copy QC headers");
-            return NULL;
+        /* Loop through properties in qclist */
+        n = cpl_propertylist_get_size(qclist);
+
+        for(i = 0; i < n; i++) {
+            prop = cpl_propertylist_get_const(qclist, i);
+            if(prop != NULL) {
+                copyme = 1;
+
+                /* Check for NaN or Inf and convert to NULL.  CPL and
+                 * CFITSIO don't allow these values in FITS headers,
+                 * so failing to do this can result in a crash if we
+                 * manage to produce these values somehow. */
+                type = cpl_property_get_type(prop);
+                if(type == CPL_TYPE_FLOAT) {
+                    if(!isfinite(cpl_property_get_float(prop))) {
+                        copyme = 0;
+                    }
+                }
+                else if(type == CPL_TYPE_DOUBLE) {
+                    if(!isfinite(cpl_property_get_double(prop))) {
+                        copyme = 0;
+                    }
+                }
+
+                /* Check for existing property with same name and
+                 * remove if present.  Failing to do this can result
+                 * in a crash if CPL infers a different data type
+                 * for the old property than what we gave the new
+                 * one. */
+                name = cpl_property_get_name(prop);
+
+                if(cpl_propertylist_has(outhdr, name)) {
+                    cpl_propertylist_erase(outhdr, name);
+                }
+
+                if(!copyme) {
+                    /* Nan or Inf might be a bug, so we emit a
+                     * warning.  It might also be an unavoidable
+                     * numerical overflow, so this shouldn't be a
+                     * fatal error. */
+                    cpl_msg_warning(cpl_func,
+                                    "FITS header %s value was not "
+                                    "finite, omitting",
+                                    name);
+                    continue;
+                }
+
+                /* Finally, we can copy it */
+                if(cpl_propertylist_copy_property(outhdr,
+                                                  qclist,
+                                                  name) != CPL_ERROR_NONE) {
+                    TIDY;
+                    cpl_error_set_message(cpl_func, cpl_error_get_code(),
+                                          "could not copy QC header: %s",
+                                          name);
+                    return NULL;
+                }
+            }
         }
     }
 
